#ifndef __TILENGINE_BI__
#define __TILENGINE_BI__

/'
* Tilengine - The 2D retro graphics engine with raster effects
* Copyright (C) 2015-2018 Marc Palacios Domenech <mailto:megamarc@hotmail.com>
* All rights reserved
*
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2 of the License, or (at your option) any later version.
*
* This library is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
* Library General Public License for more details.
*
* You should have received a copy of the GNU Library General Public
* License along with this library. If not, see <http://www.gnu.org/licenses/>.
'/

#ifdef __FB_WIN32__
 #ifndef __FB_64BIT__
   #libpath "lib/win32"
  #else
   #libpath "lib/win64"
  #endif 
#endif

#inclib "Tilengine"


' version 
#define TILENGINE_VER_MAJ	2
#define TILENGINE_VER_MIN	1
#define TILENGINE_VER_REV	1
#define TILENGINE_HEADER_VERSION ((TILENGINE_VER_MAJ shl 16) or (TILENGINE_VER_MIN shl 8) or TILENGINE_VER_REV)

#define BITVAL(n) (1 shl (n))

' tile/sprite flags. Can be none or a combination of the following: 
enum TLN_TileFlags
	FLAG_NONE		  = 0      			' no flags 
	FLAG_FLIPX		= BITVAL(15)	' horizontal flip 
	FLAG_FLIPY		= BITVAL(14)	' vertical flip 
	FLAG_ROTATE		= BITVAL(13)	' row/column flip (unsupported, Tiled compatibility) 
	FLAG_PRIORITY	= BITVAL(12)	' tile goes in front of sprite layer 
end enum

' fixed point helper 
type fix_t as long
#define FIXED_BITS	16
#define float2fix(f)	(f*(1 shl FIXED_BITS))
#define int2fix(i)		((i) shl FIXED_BITS)
#define fix2int(f)		((f) shr FIXED_BITS)
#define fix2float(f)	(f)/(1 shl FIXED_BITS)

' layer blend modes. Must be one of these and are mutually exclusive:
 
enum TLN_Blend
	BLEND_NONE    ' blending disabled 
  BLEND_MIX25   ' color averaging 1 
  BLEND_MIX50   ' color averaging 2 
  BLEND_MIX75   ' color averaging 3 
  BLEND_ADD     ' color is always brighter (simulate light effects) 
  BLEND_SUB     ' color is always darker (simulate shadow effects) 
  BLEND_MOD     ' color is always darker (simulate shadow effects) 
  BLEND_CUSTOM  ' user provided blend function with TLN_SetCustomBlendFunction() 
  MAX_BLEND
  BLEND_MIX = BLEND_MIX50
end enum

' Affine transformation parameters  
type TLN_Affine
  as single angle ' rotation in degrees 
  as single dx    ' horizontal translation 
  as single dy    ' vertical translation 
  as single sx    ' horizontal scaling 
  as single sy    ' vertical scaling 
end type

' Tile item for Tilemap access methods 
type Tile
  as ushort index ' tile index 
  as ushort flags ' attributes (FLAG_FLIPX, FLAG_FLIPY, FLAG_PRIORITY) 
end type

' frame animation definition 
type TLN_SequenceFrame
  as long index ' tile/sprite index 
  as long delay ' time delay between frames 
end type

' color strip definition 
type TLN_ColorStrip
  as long delay  ' time delay between frames 
  as ubyte first ' index of first color to cycle 
  as ubyte count ' number of colors in the cycle 
  as ubyte dir   ' direction: 0=descending, 1=ascending 
end type

' sequence info returned by TLN_GetSequenceInfo 
type TLN_SequenceInfo
  as string * (31-1) name       ' sequence name 
  as long            num_frames ' number of frames 
end type

' Sprite creation info for TLN_CreateSpriteset() 
type TLN_SpriteData
  as string * (64-1) name ' entry name 
  as long x      ' horizontal position 
  as long y      ' vertical position 
  as long w      ' width 
  as long h      ' height 
end type

' Sprite information 
type TLN_SpriteInfo
  as long w      ' width of sprite 
  as long h      ' height of sprite 
end type

' Tile information returned by TLN_GetLayerTile() 
type TLN_TileInfo
  as ushort index  ' tile index 
  as ushort flags  ' attributes (FLAG_FLIPX, FLAG_FLIPY, FLAG_PRIORITY) 
  as long row      ' row number in the tilemap 
  as long col      ' col number in the tilemap 
  as long xoffset  ' horizontal position inside the title 
  as long yoffset  ' vertical position inside the title 
  as ubyte color   ' color index at collision poas long 
  as ubyte type    ' tile type 
  as boolean empty ' cell is empty
end type

' Tileset attributes for TLN_CreateTileset() 
type TLN_TileAttributes
  as ubyte   type     ' tile type 
  as boolean priority ' priority flag set 
end type

' overlays for CRT effect 
enum TLN_Overlay
  TLN_OVERLAY_NONE       ' no overlay 
  TLN_OVERLAY_SHADOWMASK ' Shadow mask pattern 
  TLN_OVERLAY_APERTURE   ' Aperture grille pattern 
  TLN_OVERLAY_SCANLINES  ' Scanlines pattern 
  TLN_OVERLAY_CUSTOM     ' User-provided when calling TLN_CreateWindow() 
  TLN_MAX_OVERLAY
end enum

' pixel mapping for TLN_SetLayerPixelMapping() 
type TLN_PixelMap
  as short dx    ' horizontal pixel displacement 
  as short dy    ' vertical pixel displacement 
end type

type _Engine       as any
type _Tile         as any
type _Tileset      as any
type _Tilemap      as any
type _Palette      as any
type _Spriteset    as any
type _Sequence     as any
type _SequencePack as any
type _Bitmap       as any

type TLN_Engine       as _Engine ptr       ' Engine context 
type TLN_Tile         as _Tile ptr         ' Tile reference 
type TLN_Tileset      as _Tileset ptr      ' Opaque tileset reference 
type TLN_Tilemap      as _Tilemap ptr      ' Opaque tilemap reference 
type TLN_Palette      as _Palette ptr      ' Opaque palette reference 
type TLN_Spriteset    as _Spriteset ptr    ' Opaque sspriteset reference 
type TLN_Sequence     as _Sequence ptr     ' Opaque sequence reference 
type TLN_SequencePack as _SequencePack ptr ' Opaque sequence pack reference 
type TLN_Bitmap       as _Bitmap ptr       ' Opaque bitmap reference 


type SDL_Event as any

' callbacks 
type TLN_VideoCallback as sub cdecl (byval scanline as long)
type TLN_SDLCallback   as sub cdecl (byval event as SDL_Event ptr)
type TLN_BLENDCallback as function cdecl (byval src as ubyte, byval dst as ubyte) as ubyte

' Player index for input assignment functions 
enum TLN_Player
  PLAYER1  ' Player 1 
  PLAYER2  ' Player 2 
  PLAYER3  ' Player 3 
  PLAYER4  ' Player 4 
end enum

' Standard inputs query for TLN_GetInput() 
enum TLN_Input
  INPUT_NONE     ' no input 
  INPUT_UP       ' up direction 
  INPUT_DOWN     ' down direction 
  INPUT_LEFT     ' left direction 
  INPUT_RIGHT    ' right direction 
  INPUT_BUTTON1  ' 1st action button 
  INPUT_BUTTON2  ' 2nd action button 
  INPUT_BUTTON3  ' 3th action button 
  INPUT_BUTTON4  ' 4th action button 
  INPUT_BUTTON5  ' 5th action button 
  INPUT_BUTTON6  ' 6th action button 
  INPUT_START    ' Start button 

  INPUT_P1 = (PLAYER1 shl 4) ' request player 1 input (default) 
  INPUT_P2 = (PLAYER2 shl 4) ' request player 2 input 
  INPUT_P3 = (PLAYER3 shl 4) ' request player 3 input 
  INPUT_P4 = (PLAYER4 shl 4) ' request player 4 input 
  
  ' compatibility symbols for pre-1.18 input model  
  INPUT_A = INPUT_BUTTON1
  INPUT_B = INPUT_BUTTON2
  INPUT_C = INPUT_BUTTON3
  INPUT_D = INPUT_BUTTON4
  INPUT_E = INPUT_BUTTON5
  INPUT_F = INPUT_BUTTON6
end enum


' CreateWindow flags. Can be none or a combination of the following: 
enum TLN_WindowFlags 
  CWF_FULLSCREEN =  (1 shl 0)  ' create a fullscreen window 
  CWF_VSYNC      =  (1 shl 1)  ' sync frame updates with vertical retrace 
  CWF_S1         =  (1 shl 2)  ' create a window the same size as the framebuffer 
  CWF_S2         =  (2 shl 2)  ' create a window 2x the size the framebuffer 
  CWF_S3         =  (3 shl 2)  ' create a window 3x the size the framebuffer 
  CWF_S4         =  (4 shl 2)  ' create a window 4x the size the framebuffer 
  CWF_S5         =  (5 shl 2)  ' create a window 5x the size the framebuffer 
  CWF_NEAREST    =  (1 shl 6)  ' unfiltered upscaling 
end enum

' Error codes 
enum TLN_Error
  TLN_ERR_OK             ' No error 
  TLN_ERR_OUT_OF_MEMORY  ' Not enough memory 
  TLN_ERR_IDX_LAYER      ' Layer index out of range 
  TLN_ERR_IDX_SPRITE     ' Sprite index out of range 
  TLN_ERR_IDX_ANIMATION  ' Animation index out of range 
  TLN_ERR_IDX_PICTURE    ' Picture or tile index out of range 
  TLN_ERR_REF_TILESET    ' Invalid TLN_Tileset reference 
  TLN_ERR_REF_TILEMAP    ' Invalid TLN_Tilemap reference 
  TLN_ERR_REF_SPRITESET  ' Invalid TLN_Spriteset reference 
  TLN_ERR_REF_PALETTE    ' Invalid TLN_Palette reference 
  TLN_ERR_REF_SEQUENCE   ' Invalid TLN_Sequence reference 
  TLN_ERR_REF_SEQPACK    ' Invalid TLN_SequencePack reference 
  TLN_ERR_REF_BITMAP     ' Invalid TLN_Bitmap reference 
  TLN_ERR_NULL_POINTER   ' Null pointer as argument  
  TLN_ERR_FILE_NOT_FOUND ' Resource file not found 
  TLN_ERR_WRONG_FORMAT   ' Resource file has invalid format 
  TLN_ERR_WRONG_SIZE     ' A width or height parameter is invalid 
  TLN_ERR_UNSUPPORTED    ' Unsupported function 
  TLN_MAX_ERR
end enum


' Debug level 
enum TLN_LogLevel
  TLN_LOG_NONE    ' Don't pras long anything (default) 
  TLN_LOG_ERRORS  ' Pras long only runtime errors 
  TLN_LOG_VERBOSE ' Pras long everything 
end enum

extern "C"

' Basic setup and management 
declare function TLN_Init (byval hres as long, byval vres as long, byval numlayers as long, byval numsprites as long, byval numanimations as long) as TLN_Engine
declare sub      TLN_Deinit()
declare function TLN_DeleteContext (byval context as TLN_Engine) as boolean
declare function TLN_SetContext(byval context as TLN_Engine) as boolean
declare function TLN_GetContext() as TLN_Engine
declare function TLN_GetWidth() as long
declare function TLN_GetHeight() as long
declare function TLN_GetBPP() as long
declare function TLN_GetNumObjects() as ulong
declare function TLN_GetUsedMemory() as ulong
declare function TLN_GetVersion() as ulong
declare function TLN_GetNumLayers() as long
declare function TLN_GetNumSprites() as long
declare sub      TLN_SetBGColor(byval r as ubyte, byval g as ubyte, byval b as ubyte)
declare function TLN_SetBGColorFromTilemap (byval tilemap as TLN_Tilemap) as boolean
declare sub      TLN_DisableBGColor()
declare function TLN_SetBGBitmap(byval bitmap as TLN_Bitmap) as boolean
declare function TLN_SetBGPalette(byval _palette as TLN_Palette) as boolean
declare sub      TLN_SetRasterCallback(byval cb as TLN_VideoCallback)
declare sub      TLN_SetFrameCallback(byval cb as TLN_VideoCallback)
declare sub      TLN_SetRenderTarget(byval _data as ubyte ptr, byval pitch as long)
declare sub      TLN_UpdateFrame(byval _time as long)
declare sub      TLN_BeginFrame(byval _time as long)
declare function TLN_DrawNextScanline() as boolean
declare sub      TLN_SetLoadPath(byval path as const zstring ptr)
declare sub      TLN_SetCustomBlendFunction(byval cb as TLN_BLENDCallback)
declare sub      TLN_SetLogLevel(byval log_level as TLN_LogLevel)

' Error handling 
declare sub      TLN_SetLastError(byval _error as TLN_Error)
declare function TLN_GetLastError() as TLN_Error
declare function TLN_GetErrorString(byval _error as TLN_Error) as const zstring ptr

' Built-in window and input management 
declare function TLN_CreateWindow(byval overlay as const zstring ptr, byval flags as TLN_WindowFlags) as boolean
declare function TLN_CreateWindowThread(byval overlay as const zstring ptr, byval flags as TLN_WindowFlags) as boolean
declare sub      TLN_SetWindowTitle(byval title as const zstring ptr)
declare function TLN_ProcessWindow() as boolean
declare function TLN_IsWindowActive() as boolean
declare function TLN_GetInput(byval id as TLN_Input) as boolean
declare sub      TLN_EnableInput(byval player as TLN_Player, byval enable as boolean)
declare sub      TLN_AssignInputJoystick(byval player as TLN_Player, byval index as long)
declare sub      TLN_DefineInputKey(byval player as TLN_Player, byval _input as TLN_Input, byval keycode as ulong)
declare sub      TLN_DefineInputButton(byval player as TLN_Player, byval _input as TLN_Input, byval joybutton as ubyte)
declare sub      TLN_DrawFrame(byval _time as long)
declare sub      TLN_WaitRedraw()
declare sub      TLN_DeleteWindow()
declare sub      TLN_EnableBlur(byval mode as boolean)
declare sub      TLN_EnableCRTEffect(byval overlay as TLN_Overlay, byval overlay_factor as ubyte, byval threshold as ubyte, byval v0 as ubyte, byval v1 as ubyte, byval v2 as ubyte, byval v3 as ubyte, byval blur as boolean, byval glow_factor as ubyte)
declare sub      TLN_DisableCRTEffect()
declare sub      TLN_SetSDLCallback(byval cb as TLN_SDLCallback)
declare sub      TLN_Delay(byval msecs as ulong)
declare function TLN_GetTicks() as ulong
declare sub      TLN_BeginWindowFrame(byval _time as long)
declare sub      TLN_EndWindowFrame()

' Spriteset resources management for sprites 
declare function TLN_CreateSpriteset(byval bitmap as TLN_Bitmap, byval _data as TLN_SpriteData ptr, byval num_entries as long) as TLN_Spriteset
declare function TLN_LoadSpriteset(byval filename as const zstring ptr) as TLN_Spriteset
declare function TLN_CloneSpriteset(byval spriteset as TLN_Spriteset) as TLN_Spriteset
declare function TLN_GetSpriteInfo(byval spriteset as TLN_Spriteset, byval entry as long, byval info as TLN_SpriteInfo ptr) as boolean
declare function TLN_GetSpritesetPalette(byval spriteset as TLN_Spriteset) as TLN_Palette 
declare function TLN_FindSpritesetSprite(byval spriteset as TLN_Spriteset, byval _name as zstring ptr) as long
declare function TLN_SetSpritesetData(byval spriteset as TLN_Spriteset, byval entry as long, byval _data as TLN_SpriteData ptr, byval pixels as any ptr, byval pitch as long) as boolean
declare function TLN_DeleteSpriteset(byval spriteset as TLN_Spriteset) as boolean

' Tileset resources management for background layers 
declare function TLN_CreateTileset(byval numtiles as long, byval w as long, byval h as long, byval _palette as TLN_Palette, byval sp as TLN_SequencePack, byval attributes as TLN_TileAttributes ptr) as TLN_Tileset
declare function TLN_LoadTileset(byval filename as const zstring ptr) as TLN_Tileset
declare function TLN_CloneTileset(byval tileset as TLN_Tileset) as TLN_Tileset
declare function TLN_SetTilesetPixels(byval tileset as TLN_Tileset, byval entry as long, byval srcdata as ubyte ptr, byval srcpitch as long) as boolean
declare function TLN_CopyTile(byval tileset as TLN_Tileset, byval src as long, byval dst as long) as boolean
declare function TLN_GetTileWidth(byval tileset as TLN_Tileset) as long
declare function TLN_GetTileHeight(byval tileset as TLN_Tileset) as long
declare function TLN_GetTilesetPalette(byval tileset as TLN_Tileset) as TLN_Palette
declare function TLN_GetTilesetSequencePack(byval tileset as TLN_Tileset) as TLN_SequencePack
declare function TLN_DeleteTileset(byval tileset as TLN_Tileset) as boolean

' Tilemap resources management for background layers 
declare function TLN_CreateTilemap(byval rows as long, byval cols as long, byval tiles as TLN_Tile, byval bgcolor as ulong, byval tileset as TLN_Tileset) as TLN_Tilemap
declare function TLN_LoadTilemap(byval filename as const zstring ptr, byval layername as const zstring ptr) as TLN_Tilemap
declare function TLN_CloneTilemap(byval tilemap as TLN_Tilemap) as TLN_Tilemap
declare function TLN_GetTilemapRows(byval tilemap as TLN_Tilemap) as long
declare function TLN_GetTilemapCols(byval tilemap as TLN_Tilemap) as long
declare function TLN_GetTilemapTileset(byval tilemap as TLN_Tilemap) as TLN_Tileset
declare function TLN_GetTilemapTile(byval tilemap as TLN_Tilemap, byval row as long, byval col as long, byval tile as TLN_Tile) as boolean
declare function TLN_SetTilemapTile(byval tilemap as TLN_Tilemap, byval row as long, byval col as long, byval tile as TLN_Tile) as boolean
declare function TLN_CopyTiles(byval src as TLN_Tilemap, byval srcrow as long, byval srccol as long, byval rows as long, byval cols as long, byval dst as TLN_Tilemap, byval dstrow as long, byval dstcol as long) as boolean
declare function TLN_DeleteTilemap(byval tilemap as TLN_Tilemap) as boolean

' Color palette resources management for sprites and background layers 
declare function TLN_CreatePalette(byval entries as long) as TLN_Palette 
declare function TLN_LoadPalette(byval filename as const zstring ptr) as TLN_Palette
declare function TLN_ClonePalette(byval _palette as TLN_Palette) as TLN_Palette
declare function TLN_SetPaletteColor(byval _palette as TLN_Palette, byval _color as long, byval r as ubyte, byval g as ubyte, byval b as ubyte) as boolean
declare function TLN_MixPalettes(byval src1 as TLN_Palette, byval src2 as TLN_Palette, byval dst as TLN_Palette, byval factor as ubyte) as boolean
declare function TLN_AddPaletteColor(byval _palette as TLN_Palette, byval r as ubyte, byval g as ubyte, byval b as ubyte, byval start as ubyte, byval num as ubyte) as boolean
declare function TLN_SubPaletteColor(byval _palette as TLN_Palette, byval r as ubyte, byval g as ubyte, byval b as ubyte, byval start as ubyte, byval num as ubyte) as boolean
declare function TLN_ModPaletteColor(byval _palette as TLN_Palette, byval r as ubyte, byval g as ubyte, byval b as ubyte, byval start as ubyte, byval num as ubyte) as boolean
declare function TLN_GetPaletteData(byval _palette as TLN_Palette, byval index as long) as ubyte ptr
declare function TLN_DeletePalette(byval _palette as TLN_Palette) as boolean

' Bitmap management 
declare function TLN_CreateBitmap(byval w as long, byval h as long, byval bpp as long) as TLN_Bitmap
declare function TLN_LoadBitmap(byval filename as const zstring ptr) as TLN_Bitmap
declare function TLN_CloneBitmap(byval bitmap as TLN_Bitmap) as TLN_Bitmap
declare function TLN_GetBitmapPtr(byval bitmap as TLN_Bitmap, byval x as long, byval y as long) as ubyte ptr
declare function TLN_GetBitmapWidth(byval bitmap as TLN_Bitmap) as long
declare function TLN_GetBitmapHeight(byval bitmap as TLN_Bitmap) as long
declare function TLN_GetBitmapDepth(byval bitmap as TLN_Bitmap) as long
declare function TLN_GetBitmapPitch(byval bitmap as TLN_Bitmap) as long
declare function TLN_GetBitmapPalette(byval bitmap as TLN_Bitmap) as TLN_Palette
declare function TLN_SetBitmapPalette(byval bitmap as TLN_Bitmap, byval _palette as TLN_Palette) as boolean
declare function TLN_DeleteBitmap(byval bitmap as TLN_Bitmap) as boolean

' Background layers management 
declare function TLN_SetLayer(byval nlayer as long, byval tileset as TLN_Tileset, byval tilemap as TLN_Tilemap) as boolean
declare function TLN_SetLayerBitmap(byval nlayer as long, byval bitmap as TLN_Bitmap) as boolean
declare function TLN_SetLayerPalette(byval nlayer as long, byval _palette as TLN_Palette) as boolean
declare function TLN_SetLayerPosition(byval nlayer as long, byval hstart as long, byval vstart as long) as boolean
declare function TLN_SetLayerScaling(byval nlayer as long, byval xfactor as single, byval yfactor as single) as boolean
declare function TLN_SetLayerAffineTransform(byval nlayer as long, byval affine as TLN_Affine ptr) as boolean
declare function TLN_SetLayerTransform(byval nlayer as long, byval angle as single, byval dx as single, byval dy as single, byval sx as single, byval sy as single) as boolean
declare function TLN_SetLayerPixelMapping(byval nlayer as long, byval table as TLN_PixelMap ptr) as boolean
declare function TLN_SetLayerBlendMode(byval nlayer as long, byval mode as TLN_Blend, byval factor as ubyte) as boolean
declare function TLN_SetLayerColumnOffset(byval nlayer as long, byval offset as long ptr) as boolean
declare function TLN_SetLayerClip(byval nlayer as long, byval x1 as long, byval y1 as long, byval x2 as long, byval y2 as long) as boolean
declare function TLN_DisableLayerClip(byval nlayer as long) as boolean
declare function TLN_SetLayerMosaic(byval nlayer as long, byval w as long, byval h as long) as boolean
declare function TLN_DisableLayerMosaic(byval nlayer as long) as boolean
declare function TLN_ResetLayerMode(byval nlayer as long) as boolean
declare function TLN_DisableLayer(byval nlayer as long) as boolean
declare function TLN_GetLayerPalette(byval nlayer as long) as TLN_Palette
declare function TLN_GetLayerTile(byval nlayer as long, byval x as long, byval y as long, byval info as TLN_TileInfo ptr) as boolean
declare function TLN_GetLayerWidth(byval nlayer as long) as long
declare function TLN_GetLayerHeight(byval nlayer as long) as long

' Sprites management 
declare function TLN_ConfigSprite(byval nsprite as long, byval spriteset as TLN_Spriteset, byval flags as TLN_TileFlags) as boolean
declare function TLN_SetSpriteSet(byval nsprite as long, byval spriteset as TLN_Spriteset) as boolean
declare function TLN_SetSpriteFlags(byval nsprite as long, byval flags as TLN_TileFlags) as boolean
declare function TLN_SetSpritePosition(byval nsprite as long, byval x as long, byval y as long) as boolean
declare function TLN_SetSpritePicture(byval nsprite as long, byval entry as long) as boolean
declare function TLN_SetSpritePalette(byval nsprite as long, byval _palette as TLN_Palette) as boolean
declare function TLN_SetSpriteBlendMode(byval nsprite as long, byval mode as TLN_Blend, byval factor as ubyte) as boolean
declare function TLN_SetSpriteScaling(byval nsprite as long, byval sx as single, byval sy as single) as boolean
declare function TLN_ResetSpriteScaling(byval nsprite as long) as boolean
' declare function TLN_SetSpriteRotation(byval nsprite as long, byval angle as single) as boolean
' declare function TLN_ResetSpriteRotation(byval nsprite as long) as boolean
declare function TLN_GetSpritePicture(byval nsprite as long) as long
declare function TLN_GetAvailableSprite() as long
declare function TLN_EnableSpriteCollision(byval nsprite as long, byval enable as boolean) as boolean
declare function TLN_GetSpriteCollision(byval nsprite as long) as boolean
declare function TLN_DisableSprite(byval nsprite as long) as boolean
declare function TLN_GetSpritePalette(byval nsprite as long) as TLN_Palette

' Sequence resources management for layer, sprite and palette animations 
declare function TLN_CreateSequence(byval _name as const zstring ptr, byval target as long, byval num_frames as long, byval frames as TLN_SequenceFrame ptr) as TLN_Sequence
declare function TLN_CreateCycle(byval _name as const zstring ptr, byval nstrips as long, byval strips as TLN_ColorStrip ptr) as TLN_Sequence
declare function TLN_CloneSequence(byval sequence as TLN_Sequence) as TLN_Sequence
declare function TLN_GetSequenceInfo(byval sequence as TLN_Sequence, byval info as TLN_SequenceInfo ptr) as boolean
declare function TLN_DeleteSequence(byval sequence as TLN_Sequence) as boolean

' Sequence pack manager for grouping and finding sequences 
declare function TLN_CreateSequencePack() as TLN_SequencePack
declare function TLN_LoadSequencePack(byval filename as const zstring ptr) as TLN_SequencePack
declare function TLN_CloneSequencePack(byval sp as TLN_SequencePack) as TLN_SequencePack
declare function TLN_GetSequence(byval sp as TLN_SequencePack, byval index as long) as TLN_Sequence
declare function TLN_FindSequence(byval sp as TLN_SequencePack, byval _name as const zstring ptr) as TLN_Sequence
declare function TLN_GetSequencePackCount(byval sp as TLN_SequencePack) as long
declare function TLN_AddSequenceToPack(byval sp as TLN_SequencePack, byval sequence as TLN_Sequence) as boolean
declare function TLN_DeleteSequencePack(byval sp as TLN_SequencePack) as boolean

' Animation engine manager 
declare function TLN_SetPaletteAnimation(byval index as long, byval _palette as TLN_Palette, byval sequence as TLN_Sequence, byval blend as boolean) as boolean
declare function TLN_SetPaletteAnimationSource(byval index as long, byval _palette as TLN_Palette) as boolean
declare function TLN_SetTilesetAnimation(byval index as long, byval nlayer as long, byval sequence as TLN_Sequence) as boolean
declare function TLN_SetTilemapAnimation(byval index as long, byval nlayer as long, byval sequence as TLN_Sequence) as boolean
declare function TLN_SetSpriteAnimation(byval index as long, byval nsprite as long, byval sequence as TLN_Sequence, byval loops as long) as boolean
declare function TLN_GetAnimationState(byval index as long) as boolean
declare function TLN_SetAnimationDelay(byval index as long, byval delay as long) as boolean
declare function TLN_GetAvailableAnimation() as long
declare function TLN_DisableAnimation(byval index as long) as boolean


end extern ' "C"


#endif ' __TILENGINE_BI__
